Class: Docscribe::Types::Sorbet::SourceProvider
- Inherits:
-
BaseProvider
- Object
- BaseProvider
- Docscribe::Types::Sorbet::SourceProvider
- Defined in:
- lib/docscribe/types/sorbet/source_provider.rb
Overview
Sorbet provider for inline signatures present in the current Ruby source.
This provider parses the source being rewritten and indexes any leading ‘sig` declarations it can resolve through the RBS RBI prototype bridge.
Instance Method Summary collapse
Methods inherited from BaseProvider
Constructor Details
#initialize(source:, file:, collapse_generics: false) ⇒ Object
18 19 20 21 |
# File 'lib/docscribe/types/sorbet/source_provider.rb', line 18 def initialize(source:, file:, collapse_generics: false) super(collapse_generics: collapse_generics) load_from_string(source, label: file) end |